<?php

// QuickTalk forum 1.2 build:20070219

/*
 * Database class
 *
 * NOTE:
 * the function row_count have been removed because it is not supported by odbc
 *
 */

class cDB
{
   var $db_connect="";
   var $db_close="";
   var $db_select_db="";
   var $db_query="";
   var $db_fetch_array="";
   var $db_num_rows=""; 
   var $host;
   var $type;
   var $database;
   var $user;
   var $user_ip="";
   var $password;
   var $port;
   var $dsn;   
   var $sql;   
   var $con; // variable for connection id
   var $con_string; // variable for connection string
   var $query_id; // variable for query id   
   var $error_nr;
   var $error=""; 
   var $debug=false; // debug mode off
   
   /**
   * Constructor of class - Initializes class and connects to the database
   * @param string $database_type the name of the database
   * @param string $host the host of the database
   * @param string $database the name of the database
   * @param string $user the name of the user for the database
   * @param string $password the passord of the user for the database
   * @desc Constructor of class - Initializes class and connects to the database.
   *
   *  You can use this shortcuts for the database type:
   *
   *     ifx -> INFORMIX
   *     mssql -> Microsoft SQL Server
   *     mysql -> MySQL
   *     odbc -> ODBC for MS-Access 95, MS-Access XP
   *     pg -> PostgreSQL
   */
   function cDB($type,$host,$database,$user,$password,$port=false,$dsn=false)
   {
      $this->type     = strtolower($type);
      $this->host     = $host;
      $this->database = $database;
      $this->user     = $user;
      $this->user_ip  = $_SERVER["REMOTE_ADDR"];
      $this->password = $password;
      $this->port     = $port;
      $this->dsn      = $dsn;
      
      // check type
      $arrTypes=array("ifx","mssql","mysql","odbc","pg");
      if ( !in_array($this->type,$arrTypes) )
      {
         $this->halt("Database type ({$this->type}) not supported");
   return false;
      }
      
      // Setting database type
      if( $this->type=="odbc" )
      {
            $this->db_connect="odbc_connect";
            $this->db_close="odbc_close";
            $this->db_select_db="odbc_select_db";
            $this->db_query="odbc_exec";
            $this->db_fetch_array="odbc_fetch_row";
      }
      else
      {
            $this->db_connect=$this->type."_connect";
            $this->db_close=$this->type."_close";
            $this->db_select_db=$this->type."_select_db";
            $this->db_query=$this->type."_query";
            $this->db_fetch_array=$this->type."_fetch_array";
      }
      $this->db_num_rows=$this->type."_num_rows";
      
      // Connect
      return $this->connect();
   }
   
   /**
   * This function connects the database
   * @return boolean $is_connected Returns true if connection was successful otherwise false
   * @desc This function connects to the database which is set in the constructor
   */
   function connect()
   {
      // Selecting connection function and connecting 
      if($this->con=="")
      {
         if ( $this->type=="ifx" )
         {
            $this->con=call_user_func($this->db_connect,$this->database."@".$this->host,$this->user,$this->password);
         }
         else if ( $this->type=="mysql" )
         {
            // With port
            if ( $this->port )
            {
               $this->con=call_user_func($this->db_connect,$this->host.":".$this->port,$this->user,$this->password);
            }
            else
            {   // Without port
               $this->con=call_user_func($this->db_connect,$this->host,$this->user,$this->password);
            }
         }
         else if ( $this->type=="mssql" )
         {
            $this->con=call_user_func($this->db_connect,$this->host,$this->user,$this->password);
         }
         else if ( $this->type=="odbc" )
         {
            $this->con=call_user_func($this->db_connect,$this->dsn,$this->user,$this->password);
         }
         else if ( $this->type=="pg" )
         {
            // With port
            if( !$this->port )
            {
               $this->con=call_user_func($this->db_connect,"host=".$this->host." port=".$this->port." dbname=".$this->database." user=".$this->user." password=".$this->password);
            }
            // Without port
            else
            {
               $this->con=call_user_func($this->db_connect,"host=".$this->host." dbname=".$this->database." user=".$this->user." password=".$this->password);
            }
         }
         
         if ( !$this->con )
         {
            $this->halt("Wrong connection data! Can't establish connection to host.");
            return false;
         }
         else
         {
            if ( ($this->type!="odbc") && ($this->type!="pg") )
            {
               if ( !call_user_func($this->db_select_db,$this->database,$this->con) )
               {
                  $this->halt("Wrong database data! Can't select database.");
                  return false;
               }
               else
               {
                  return true;
               }
            }
         }
      }
      else
      {
         $this->halt("Already connected to database.");
         return false;
      }
   }
   
   /**
   * This function disconnects from the database
   * @desc This function disconnects from the database
   */
   function disconnect()
   {
      if ( @call_user_func($this->db_close,$this->con) )
      {
         return true;
      }
      else
      {
         $this->halt("Not connected yet");
         return false;
      }
   }
   
   /**
   * This function starts the sql query
   * @param string $sql_statement the sql statement
   * @return boolean $successfull returns false on errors otherwise true
   * @desc This function disconnects from the database
   */
   function Query($sql_statement)
   {
      $this->sql=$sql_statement;

      if ( $this->debug ) printf("SQL statement: %s\n",$this->sql);

      if ( $this->type=="odbc" )
      {
         // ODBC
         // odbc requires to change " into '
         $this->sql = str_replace("\"","'",$this->sql);
         
         $this->query_id = call_user_func($this->db_query,$this->con,$this->sql);
         if( !$this->query_id )
         {
            $this->halt("No database connection exists or invalid query ".$this->sql);
            return false;
         }
         else
         {
            return true;
         }
      }
      else if($this->type=="pg")
      {
         // PG
         // PG requires to change " into '
         $this->sql = str_replace("\"","'",$this->sql);

         // PG
         $this->query_id = call_user_func($this->db_query,$this->con,$this->sql);
         if ( !$this->query_id )
         {
            $this->halt("No database connection exists or invalid query ".$this->sql);
            return false;
         }
         else
         {
            return true;
         }
      }     
      else
      {
         // All other databases
         $this->query_id = call_user_func($this->db_query,$this->sql,$this->con);
         if ( !$this->query_id )
         {
            $this->halt("No database connection exists or invalid query ".$this->sql);
            return false;
         }
         else
         {
            return true;
         }
      }
   }
   
   /**
   * This function returns a row of the resultset
   * @return array $row the row as array or false if there is no more row
   * @desc This function returns a row of the resultset
   */
   function Getrow()
   {
      if ( $this->type=="odbc" )
      {
         // ODBC database
         if ( $row=call_user_func($this->db_fetch_array,$this->query_id) )
         {
            
            for ($i=1; $i<=odbc_num_fields($this->query_id); $i++)
            {
               $fieldname=odbc_field_name($this->query_id,$i);
               $row_array[$fieldname]=odbc_result($this->query_id,$i);
            }
            return $row_array;
         }
         else
         {
            return false;
         }
      }
      else
      {
         // All other databases
         $row=call_user_func($this->db_fetch_array,$this->query_id);
         return $row;
      }
   }
   
   /**
   * This function returns all tables of the database in an array
   * @return array $tables all tables of the database in an array
   * @desc This function returns all tables of the database in an array
   */
   function get_tables()
   {
      if ( $this->type=="odbc" )
      {
         // ODBC databases
         $tablelist=odbc_tables($this->con);
         
         for($i=0;odbc_fetch_row($tablelist);$i++)
         {
            $tables[$i]=odbc_result($tablelist,3);
         }
         return $tables;
      }
      else
      {
         // All other databases
         $tables = "";
         $sql="SHOW TABLES";
         $this->query_id($sql);
         for($i=0;$data=$this->Getrow();$i++)
         {
            $tables[$i]=$data['Tables_in_'.$this->database];
         }
         return $tables;
      }
   }
   
   /*
    * This function print the error message
    */
   function halt($message)
   {
      $this->error = $message;
      printf("Database error: %s Session halted.", $message);
      if ( $this->error_nr!="" && $this->error!="" )
      {
         printf("MySQL Error: %s (%s)\n",$this->error_nr,$this->error);
      }
   }
   
   function debug_mode($debug=true)
   {
      $this->debug=$debug;
   }
}

?>